/* 
  Multiple batch example (retrieves one giant set of data)
  https://developer.salesforce.com/docs/atlas.en-us.api_asynch.meta/api_asynch/asynch_api_code_curl_walkthrough_pk_chunking.htm
*/

{i/sfdctt.i}

DEFINE TEMP-TABLE ttBatches NO-UNDO
  FIELD batchID     AS CHARACTER 
  FIELD batchStatus AS CHARACTER 
  INDEX idxMain batchStatus. 

DEFINE VARIABLE sfdcBULK     AS SFDC.SFDCBULK NO-UNDO.
DEFINE VARIABLE htt          AS HANDLE        NO-UNDO. 
DEFINE VARIABLE jobId        AS CHARACTER     NO-UNDO. 
DEFINE VARIABLE batchId      AS CHARACTER     NO-UNDO. 
DEFINE VARIABLE i            AS INTEGER       NO-UNDO. 
DEFINE VARIABLE batchStatus  AS CHARACTER     NO-UNDO. 
DEFINE VARIABLE jobState     AS CHARACTER     NO-UNDO. 
DEFINE VARIABLE batchIdList  AS CHARACTER     NO-UNDO. 
DEFINE VARIABLE batchIdList1 AS CHARACTER     NO-UNDO. 

ETIME (TRUE).
OUTPUT TO "Logs/bulk.txt".
htt = TEMP-TABLE ttCustomer:HANDLE. 

sfdcBULK = NEW SFDC.SFDCBULK().
sfdcBULK:contentType = "CSV".

/* enable PKChunking */ 
jobId = sfdcBULK:CreateJob("query", "Account", TRUE ). 
PUT UNFORMATTED 
  NOW " JobID: "jobId SKIP.
 
/* create a primary batch - it doesn't do any real work and is never marked processed */ 
batchId = sfdcBULK:CreateBatch(htt, jobId).
PUT UNFORMATTED 
  NOW " Primary BatchID: " batchID SKIP.

/* get back the list of batches that are automatically created that will return the results */ 
batchIDList = sfdcBULK:GetBatchIDs(jobId).
PUT UNFORMATTED 
  NOW " Batches?: " batchIDList SKIP. 

DO i = 1 TO NUM-ENTRIES (batchIDList):
  CREATE ttBatches.
  ASSIGN 
    ttBatches.batchID     = ENTRY (i, batchIDList)
    ttBatches.batchStatus = "Queued".
END.

polloopalooza:
REPEAT:  
  IF NOT CAN-FIND (FIRST ttBatches WHERE ttBatches.batchStatus = "Queued") THEN  
    LEAVE polloopalooza.

  FOR EACH ttBatches
    WHERE ttBatches.batchStatus = "Queued":
    batchStatus = sfdcBULK:CheckBatchStatus(jobId, ttBatches.batchID). 
    IF batchStatus = "failed" OR batchStatus = "InvalidBatch" THEN  
    DO:
      MESSAGE  "Site Batch Status = " + batchStatus
        VIEW-AS ALERT-BOX.
      RETURN.     
    END.

    IF batchStatus EQ "Completed" THEN
    DO:
      ASSIGN 
        ttBatches.batchStatus = "Completed".
      PUT UNFORMATTED ttBatches.batchID " Completed " SKIP . 
    END.
    
    PUT UNFORMATTED NOW " " ttBatches.batchID " Batch not finished." SKIP.  
    System.Threading.Thread:Sleep(3000).
  END.         
  
  /* you must pause at least a second before checking again or else SFDC will return
     the previous batch status */ 
  System.Threading.Thread:Sleep(3000).
END. /* pollLoop */

PUT UNFORMATTED NOW " Site Batch completed" SKIP.

MESSAGE "huh"
  VIEW-AS ALERT-BOX.

RETURN. 
sfdcBULK:retrieveBatchToTT (htt,jobId,batchId).
jobState = sfdcBULK:CloseJob(jobId).

FOR EACH ttCustomer:
  i = i + 1.
  DISPLAY ttCustomer.id ttcustomer.custName  FORMAT "x(44)".
END.
OUTPUT CLOSE. 

MESSAGE 
  "Records: " i SKIP 
  STRING (ETIME / 1000)
  VIEW-AS ALERT-BOX.

CATCH e AS Progress.Lang.Error:
  MESSAGE "An Error Occurred: " (e:GetMessage (1))
    VIEW-AS ALERT-BOX. 
END.    
  