/*------------------------------------------------------------------------
   File        : SFDCREST
   Purpose     : 
   Syntax      : 
   Description : 
   Author(s)   : gerry.winning
   Created     : Tue Jan 31 16:04:07 EST 2012
   Notes       : 
 ---------------------------------------------------------------------- */
 
USING Progress.Lang.*.

BLOCK-LEVEL ON ERROR UNDO, THROW.
 
CLASS SFDC.SFDCBulk: 
  /*------------------------------------------------------------------------------
      Purpose:  																	  
      Notes:  																	  
  ------------------------------------------------------------------------------*/

  {i/SFDCTT.i}
  DEFINE VARIABLE protocolType AS System.Net.SecurityProtocolType NO-UNDO.   
      
	

  DEFINE PUBLIC PROPERTY AccessToken AS CHARACTER NO-UNDO 
    GET.
    SET. 

  DEFINE PUBLIC PROPERTY ContentType AS CHARACTER NO-UNDO 
    GET.
    SET. 

  DEFINE PUBLIC PROPERTY RestURL AS CHARACTER NO-UNDO 
    GET.
    SET. 
        
  DEFINE PUBLIC PROPERTY ServerURL AS CHARACTER NO-UNDO 
    GET.
    SET. 


  /*------------------------------------------------------------------------------
      Purpose:  																	  
      Notes:  																	  
  ------------------------------------------------------------------------------*/

  CONSTRUCTOR PUBLIC SFDCBulk ():
    SUPER ().

    
    /* the following makes the webClient TLS 1.1 and 1.2 enabled for the entire session */
    /* TLS 1.2 is not enabled by default.  SFDC will be TLS 1.2 ONLY next year. */
    protocolType = CAST(
      Progress.Util.EnumHelper:Or(
      System.Net.SecurityProtocolType:Tls12, System.Net.SecurityProtocolType:Tls11),
      System.Net.SecurityProtocolType).
    protocolType = CAST(
      Progress.Util.EnumHelper:Or(protocolType, System.Net.SecurityProtocolType:Tls),
      System.Net.SecurityProtocolType).
    System.Net.ServicePointManager:SecurityProtocol = THIS-OBJECT:protocolType.
    
    RETURN. 
  END CONSTRUCTOR.


  /*------------------------------------------------------------------------------
      Purpose:  																	  
      Notes:  																	  
  ------------------------------------------------------------------------------*/

  METHOD PUBLIC CHARACTER CheckBatchStatus( pJobId AS CHARACTER, pBatchId AS CHARACTER ):
    DEFINE VARIABLE vStatus      AS CHARACTER                      NO-UNDO.
    DEFINE VARIABLE vURL         AS CHARACTER                      NO-UNDO. 
    DEFINE VARIABLE stateMessage AS CHARACTER                      NO-UNDO. 
    DEFINE VARIABLE webClient    AS System.Net.WebClient           NO-UNDO. 
    DEFINE VARIABLE xDoc         AS System.Xml.XmlDocument         NO-UNDO.   
    DEFINE VARIABLE nsManager    AS System.Xml.XmlNamespaceManager NO-UNDO.   
    DEFINE VARIABLE lcResult     AS LONGCHAR                       NO-UNDO.
      
    /* Call the web service to get the XML response containing the status */
    vURL = THIS-OBJECT:ServerURL + restURL + 
      "/job/" + pJobId + "/batch/" + pBatchId + "?antiCache=" + STRING (NOW).  
    webClient = NEW System.Net.WebClient().
    webClient:Encoding = System.Text.Encoding:UTF8.
    webClient:Headers:Set("X-SFDC-Session", THIS-OBJECT:AccessToken).
    lcResult = webClient:DownloadString (vURL). 

    /* parse the returned xml to status (the field is "state" in the xml) */
    xDoc = NEW System.XML.XMLDocument().
    xDoc:LoadXml(lcResult).
    nsManager = NEW System.XML.XmlNamespaceManager(xDoc:NameTable).
    nsManager:AddNamespace("dataload", "http://www.force.com/2009/06/asyncapi/dataload").    
    vStatus =  xdoc:SelectSingleNode("//dataload:state", nsManager):InnerText.

    /* if the status is "failed", there will be an extra node with the error named "stateMessage" */
    IF vStatus = "Failed" THEN
    DO:
      stateMessage =  xdoc:SelectSingleNode("//dataload:stateMessage", nsManager):InnerText.      
      PUT UNFORMATTED NOW " Batch Failed: " stateMessage SKIP .
    END.
 
    RETURN vStatus.

  END METHOD.

  /*------------------------------------------------------------------------------
      Purpose:  																	  
      Notes:  																	  
  ------------------------------------------------------------------------------*/

  METHOD PUBLIC VOID CloseJob( pJobIds AS CHARACTER  ):
  /*        DEFINE VARIABLE vi AS INTEGER NO-UNDO.*/
  /*        DO vi = 1 TO NUM-ENTRIES (pJobIds):   */
  /*            RUN CloseJob IN hSFDCBatchAPI (   */
  /*                INPUT cSFDCLib:serverURL,     */
  /*                INPUT cSFDCLib:accessToken,   */
  /*                INPUT ENTRY (vi,pJobIds)).    */
  /*        END.                                  */
  END METHOD.

  /*------------------------------------------------------------------------------
      Purpose:  																	  
      Notes:  																	  
  ------------------------------------------------------------------------------*/

  METHOD PUBLIC CHARACTER CreateBatch( SFDCTable AS CHARACTER, hTT AS HANDLE, pJobId AS CHARACTER    ):
    DEFINE VARIABLE batchId   AS CHARACTER                      NO-UNDO.
    DEFINE VARIABLE vosql     AS CHARACTER                      NO-UNDO.
    DEFINE VARIABLE vURL      AS CHARACTER                      NO-UNDO. 
    DEFINE VARIABLE webClient AS System.Net.Webclient           NO-UNDO.
    DEFINE VARIABLE xDoc      AS System.XML.XmlDocument         NO-UNDO.
    DEFINE VARIABLE nsManager AS System.Xml.XmlNamespaceManager NO-UNDO. 
    DEFINE VARIABLE lcResult  AS LONGCHAR                       NO-UNDO.
		
    /*        vosql = buildosql(hTT, SFDCTable).*/
    vURL = THIS-OBJECT:ServerURL + restURL + "/job/" + pJobId + "/batch".
  
    webClient = NEW System.Net.WebClient().
    webClient:Encoding = System.Text.Encoding:UTF8.
    webClient:Headers:Set("X-SFDC-Session", THIS-OBJECT:AccessToken).
    
    CASE THIS-OBJECT:contentType:
      WHEN "CSV" THEN
        webClient:Headers:Set("Content-Type","text/csv; charset=UTF-8").
      WHEN "JSON" THEN
        webClient:Headers:Set("Content-Type","application/json").
      OTHERWISE
      webClient:Headers:Set("Content-Type","application/xml; charset=UTF-8").
    END CASE.
    lcResult = webClient:UploadString (vURL, vOSQL).  
    
    xDoc = NEW System.XML.XMLDocument().
    xDoc:LoadXml(lcResult).
    nsManager = NEW System.XML.XmlNamespaceManager(xDoc:NameTable).
    nsManager:AddNamespace("dataload", "http://www.force.com/2009/06/asyncapi/dataload").    
    batchId =  xdoc:SelectSingleNode("//dataload:id", nsManager):InnerText.
      
    RETURN batchId.

  END METHOD.

  METHOD PUBLIC CHARACTER CreateJob(pOperation AS CHARACTER, pObject AS CHARACTER  ):
    DEFINE VARIABLE JobId      AS CHARACTER                      NO-UNDO.
    DEFINE VARIABLE vURL       AS CHARACTER                      NO-UNDO.
    DEFINE VARIABLE xmlRequest AS LONGCHAR                       NO-UNDO.
    DEFINE VARIABLE webClient  AS System.Net.WebClient           NO-UNDO.
    DEFINE VARIABLE lcResult   AS LONGCHAR                       NO-UNDO. 
    DEFINE VARIABLE xDoc       AS System.XML.XmlDocument         NO-UNDO.
    DEFINE VARIABLE nsManager  AS System.Xml.XmlNamespaceManager NO-UNDO. 
    
    xmlRequest = '<?xml version="1.0" encoding="UTF-8"?> ' +
      '<jobInfo xmlns="http://www.force.com/2009/06/asyncapi/dataload"> ' +
      '<operation>' + pOperation + '</operation> ' +
      '<object>' + pObject + '</object> ' +
      '<concurrencyMode>' + "Parallel" + '</concurrencyMode> ' +
      '<contentType>' + contentType + '</contentType> ' +
      '</jobInfo>'.

    vURL = ServerURL + restURL + "/job".
    webClient = NEW System.Net.WebClient().
    webClient:Encoding = System.Text.Encoding:UTF8.
    webClient:Headers:Set("X-SFDC-Session", AccessToken).
    webClient:Headers:Set("Content-Type","application/xml; charset=UTF-8").
  
    lcResult = webClient:UploadString (vURL, xmlRequest).  
    xDoc = NEW System.XML.XMLDocument().
    xDoc:LoadXml(lcResult).
    nsManager = NEW System.XML.XmlNamespaceManager(xDoc:NameTable).
    nsManager:AddNamespace("dataload", "http://www.force.com/2009/06/asyncapi/dataload").    
    jobId =  xdoc:SelectSingleNode("//dataload:id", nsManager):InnerText.
    
    RETURN jobId.
     
  END METHOD.


  /*------------------------------------------------------------------------------
   Purpose:
   Notes:
  ------------------------------------------------------------------------------*/

  METHOD PUBLIC CHARACTER GetBatchIDs( pJobId AS CHARACTER  ):
		
    DEFINE VARIABLE batchIds  AS CHARACTER                      NO-UNDO.
    DEFINE VARIABLE vURL      AS CHARACTER                      NO-UNDO. 
    DEFINE VARIABLE webClient AS System.Net.Webclient           NO-UNDO.
    DEFINE VARIABLE xDoc      AS System.XML.XmlDocument         NO-UNDO.
    DEFINE VARIABLE nsManager AS System.Xml.XmlNamespaceManager NO-UNDO. 
    DEFINE VARIABLE xmlNodes  AS System.Xml.XmlNodeList         NO-UNDO. .
    DEFINE VARIABLE lcResult AS LONGCHAR NO-UNDO.

    vURL = ServerURL + restURL + 
      "/job/" + pJobId + "/batch".
  
    webClient = NEW System.Net.WebClient().
    webClient:Encoding = System.Text.Encoding:UTF8.
    webClient:Headers:Set("X-SFDC-Session", AccessToken).
    CASE contentType:
      WHEN "CSV" THEN
        webClient:Headers:Set("Content-Type","text/csv; charset=UTF-8").
      WHEN "JSON" THEN 
        webClient:Headers:Set("Content-Type","application/json").
      OTHERWISE 
      webClient:Headers:Set("Content-Type","application/xml; charset=UTF-8").
    END CASE. 
    lcResult = webClient:DownloadString (vURL).  
    
    xDoc = NEW System.XML.XMLDocument().
    xDoc:LoadXml(lcResult).
    nsManager = NEW System.XML.XmlNamespaceManager(xDoc:NameTable).
    nsManager:AddNamespace("dataload", "http://www.force.com/2009/06/asyncapi/dataload").    
    xmlNodes = xDoc:SelectNodes("//dataload:batchInfo", nsmanager). 

    PUT UNFORMATTED  xmlNodes:Count SKIP.

    RETURN batchIds.
  END METHOD.


  /*------------------------------------------------------------------------------
      Purpose:  																	  
      Notes:  																	  
  ------------------------------------------------------------------------------*/

  METHOD PUBLIC VOID retrieveBatch( hTT AS HANDLE, pJobId AS CHARACTER, pBatchId AS CHARACTER  ):
    DEFINE VARIABLE lcResult      AS LONGCHAR                       NO-UNDO.
    DEFINE VARIABLE vURL          AS CHARACTER                      NO-UNDO.
    DEFINE VARIABLE vName         AS CHARACTER                      NO-UNDO.
    DEFINE VARIABLE webClient     AS System.Net.WebClient           NO-UNDO. 
    DEFINE VARIABLE xDoc          AS System.XML.XmlDocument         NO-UNDO.
    DEFINE VARIABLE nsManager     AS System.Xml.XmlNamespaceManager NO-UNDO. 
    DEFINE VARIABLE xmlNodes      AS System.Xml.XmlNodeList         NO-UNDO. 
    DEFINE VARIABLE resultID      AS CHARACTER                      NO-UNDO. 
    DEFINE VARIABLE delimitedFile AS Utility.DelimitedFile          NO-UNDO.
 
    webClient = NEW System.Net.WebClient().

    /*        vURL = cSFDCLib:serverURL + WinOS:restURL + "/job/" + pJobId + "/batch/" + pBatchId + "/result/".*/
  
    webClient = NEW System.Net.WebClient().
    webClient:Encoding = System.Text.Encoding:UTF8.
    webClient:Headers:Set("X-SFDC-Session", THIS-OBJECT:accessToken).
    lcResult = webClient:DownloadString (vURL).  

    xDoc = NEW System.XML.XMLDocument().
    xDoc:LoadXml(lcResult).
    nsManager = NEW System.XML.XmlNamespaceManager(xDoc:NameTable).
    nsManager:AddNamespace("dataload", "http://www.force.com/2009/06/asyncapi/dataload").    
    resultID =  xdoc:SelectSingleNode("//dataload:result", nsManager):InnerText.

    vURL =  THIS-OBJECT:serverURL + restURL + 
      "/job/" + pJobId + "/batch/" + pBatchId + "/result/" + resultId. 
    webClient = NEW System.Net.WebClient().
    webClient:Encoding = System.Text.Encoding:UTF8.
    webClient:Headers:Set("X-SFDC-Session", THIS-OBJECT:accessToken).
    lcResult = webClient:DownloadString (vURL).  

    COPY-LOB lcResult TO FILE hTT:SERIALIZE-NAME + "." + THIS-OBJECT:contentType NO-CONVERT .

    /* I think this could be done better, maybe not
       we pull CSV for the lineitem table and it must be parsed into the TT because the xml
       for the lineitem table was too big an blew up the read-xml statement
       all other tables are xml that can just be loaded with read-xml */
    IF htt:SERIALIZE-NAME = "ttSFDCLineItem" THEN 
    DO: 
      delimitedFile = NEW Utility.DelimitedFile(lcResult). 
      delimitedFile:copyToTT(TABLE-HANDLE hTT BY-REFERENCE ) .  
    END.
    ELSE 
    DO: 
      vName = hTT:SERIALIZE-NAME .
      hTT:SERIALIZE-NAME = "Records".
      hTT:READ-XML ("LongChar", lcResult, ?, ?, ?)  .
      hTT:SERIALIZE-NAME = vName.
    END.
    RETURN.

  END METHOD.

END CLASS.
